@@ -36,9 +36,9 @@
 #include <X11/Xlib.h>
 #include <X11/keysymdef.h>
 
-#include <compiz.h>
-#include <scale.h>
-#include <text.h>
+#include <compiz-core.h>
+#include <compiz-scale.h>
+#include <compiz-text.h>
 
 #include "scalefilter_options.h"
 
@@ -73,6 +73,8 @@ typedef struct _ScaleFilterDisplay {
     XIM xim;
     XIC xic;
 
+    Bool textAvailable;
+
     HandleEventProc       handleEvent;
     HandleCompizEventProc handleCompizEvent;
 } ScaleFilterDisplay;
@@ -88,16 +90,17 @@ typedef struct _ScaleFilterScreen {
 } ScaleFilterScreen;
 
 #define GET_FILTER_DISPLAY(d)				          \
-    ((ScaleFilterDisplay *) (d)->privates[displayPrivateIndex].ptr)
+    ((ScaleFilterDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define FILTER_DISPLAY(d)		          \
     ScaleFilterDisplay *fd = GET_FILTER_DISPLAY (d)
 
 #define GET_FILTER_SCREEN(s, fd)				              \
-    ((ScaleFilterScreen *) (s)->privates[(fd)->screenPrivateIndex].ptr)
+    ((ScaleFilterScreen *) (s)->base.privates[(fd)->screenPrivateIndex].ptr)
 
-#define FILTER_SCREEN(s)						               \
-    ScaleFilterScreen *fs = GET_FILTER_SCREEN (s, GET_FILTER_DISPLAY (s->display))
+#define FILTER_SCREEN(s)		                    \
+    ScaleFilterScreen *fs = GET_FILTER_SCREEN (s,           \
+			    GET_FILTER_DISPLAY (s->display))
 
 static void
 scalefilterFreeFilterText (CompScreen *s)
@@ -129,6 +132,7 @@ scalefilterRenderFilterText (CompScreen *s)
     char           buffer[2 * MAX_FILTER_STRING_LEN];
 
     FILTER_SCREEN (s);
+    FILTER_DISPLAY (s->display);
 
     if (!fs->filterInfo)
 	return;
@@ -160,8 +164,11 @@ scalefilterRenderFilterText (CompScreen *s)
     if (fs->filterInfo->filterStringLength == 0)
 	return;
 
-    tA.maxwidth = x2 - x1 - (2 * scalefilterGetBorderSize (s));
-    tA.maxheight = y2 - y1 - (2 * scalefilterGetBorderSize (s));
+    if (!fd->textAvailable)
+	return;
+
+    tA.maxWidth = x2 - x1 - (2 * scalefilterGetBorderSize (s));
+    tA.maxHeight = y2 - y1 - (2 * scalefilterGetBorderSize (s));
     tA.screen = s;
     tA.size = scalefilterGetFontSize (s);
     tA.color[0] = scalefilterGetFontColorRed (s);
@@ -220,23 +227,25 @@ scalefilterDrawFilterText (CompScreen *s,
 {
     FILTER_SCREEN (s);
 
-    GLboolean wasBlend;
-    GLint     oldBlendSrc, oldBlendDst;
-    int       ox1, ox2, oy1, oy2;
+    GLboolean  wasBlend;
+    GLint      oldBlendSrc, oldBlendDst;
+    int        k;
+    int        ox1, ox2, oy1, oy2;
+    float      width, height, border;
+    float      x, y;
+    CompMatrix *m;
 
-    float width = fs->filterInfo->textWidth;
-    float height = fs->filterInfo->textHeight;
-    float border = scalefilterGetBorderSize (s);
+    width = fs->filterInfo->textWidth;
+    height = fs->filterInfo->textHeight;
+    border = scalefilterGetBorderSize (s);
 
     ox1 = output->region.extents.x1;
     ox2 = output->region.extents.x2;
     oy1 = output->region.extents.y1;
     oy2 = output->region.extents.y2;
-    float x = ox1 + ((ox2 - ox1) / 2) - (width / 2);
-    float y = oy1 + ((oy2 - oy1) / 2) + (height / 2);
 
-    x = floor (x);
-    y = floor (y);
+    x = floor (ox1 + ((ox2 - ox1) / 2) - (width / 2));
+    y = floor (oy1 + ((oy2 - oy1) / 2) + (height / 2));
 
     wasBlend = glIsEnabled (GL_BLEND);
     glGetIntegerv (GL_BLEND_SRC, &oldBlendSrc);
@@ -265,7 +274,7 @@ scalefilterDrawFilterText (CompScreen *s,
 #define CORNER(a,b) \
     for (k = a; k < b; k++) \
     {\
-	float rad = k* (3.14159 / 180.0f);\
+	float rad = k * (3.14159 / 180.0f);\
 	glVertex2f (0.0f, 0.0f);\
 	glVertex2f (cos (rad) * border, sin (rad) * border);\
 	glVertex2f (cos ((k - 1) * (3.14159 / 180.0f)) * border, \
@@ -273,8 +282,6 @@ scalefilterDrawFilterText (CompScreen *s,
     }
 
     /* Rounded corners */
-    int k;
-
     glTranslatef (border, border, 0.0f);
     glBegin (GL_TRIANGLES);
     CORNER (180, 270) glEnd();
@@ -304,17 +311,17 @@ scalefilterDrawFilterText (CompScreen *s,
 
     enableTexture (s, &fs->filterInfo->textTexture, COMP_TEXTURE_FILTER_GOOD);
 
-    CompMatrix *m = &fs->filterInfo->textTexture.matrix;
+    m = &fs->filterInfo->textTexture.matrix;
 
     glBegin (GL_QUADS);
 
-    glTexCoord2f (COMP_TEX_COORD_X(m, 0),COMP_TEX_COORD_Y(m ,0));
+    glTexCoord2f (COMP_TEX_COORD_X (m, 0), COMP_TEX_COORD_Y (m ,0));
     glVertex2f (x, y - height);
-    glTexCoord2f (COMP_TEX_COORD_X(m, 0),COMP_TEX_COORD_Y(m, height));
+    glTexCoord2f (COMP_TEX_COORD_X (m, 0), COMP_TEX_COORD_Y (m, height));
     glVertex2f (x, y);
-    glTexCoord2f (COMP_TEX_COORD_X(m, width),COMP_TEX_COORD_Y(m, height));
+    glTexCoord2f (COMP_TEX_COORD_X (m, width), COMP_TEX_COORD_Y (m, height));
     glVertex2f (x + width, y);
-    glTexCoord2f (COMP_TEX_COORD_X(m, width),COMP_TEX_COORD_Y(m, 0));
+    glTexCoord2f (COMP_TEX_COORD_X (m, width), COMP_TEX_COORD_Y (m, 0));
     glVertex2f (x + width, y - height);
 
     glEnd ();
@@ -365,9 +372,6 @@ scalefilterRelayout (CompScreen *s)
 
     SCALE_DISPLAY (s->display);
 
-    sd->selectedWindow = None;
-    sd->hoveredWindow = None;
-
     action = &sd->opt[SCALE_DISPLAY_OPTION_RELAYOUT].value.action;
 
     o[0].type    = CompOptionTypeInt;
@@ -384,11 +388,12 @@ scalefilterRelayout (CompScreen *s)
 static void
 scalefilterInitFilterInfo (CompScreen *s)
 {
+    ScaleFilterInfo *info;
+
     FILTER_SCREEN (s);
     SCALE_SCREEN (s);
 
-    ScaleFilterInfo *info = fs->filterInfo;
-
+    info = fs->filterInfo;
     memset (info->filterString, 0, sizeof (info->filterString));
     info->filterStringLength = 0;
 
@@ -556,7 +561,7 @@ scalefilterHandleKeyPress (CompScreen *s,
     /* set the event type invalid if we
        don't want other plugins see it */
     if (dropKeyEvent)
-	event->type = LASTEvent+1;
+	event->type = LASTEvent + 1;
 
     if (needRelayout)
     {
@@ -603,8 +608,8 @@ scalefilterHandleEvent (CompDisplay *d,
 
 static void
 scalefilterHandleCompizEvent (CompDisplay *d,
-	 		      char        *pluginName,
-	 		      char        *eventName,
+	 		      const char  *pluginName,
+	 		      const char  *eventName,
 	 		      CompOption  *option,
 	 		      int         nOption)
 {
@@ -632,18 +637,14 @@ scalefilterHandleCompizEvent (CompDisplay *d,
 		matchInit (&fs->scaleMatch);
 		matchCopy (&fs->scaleMatch, ss->currentMatch);
 		matchUpdate (d, &fs->scaleMatch);
-		fs->matchApplied = FALSE;
+	    }
+	    else if (fs->filterInfo)
+	    {
+    		ss->currentMatch = fs->filterInfo->origMatch;
+		scalefilterFiniFilterInfo (s, TRUE);
 	    }
 
-	    if (!activated)
-	    {
-		if (fs->filterInfo)
-		{
-		    ss->currentMatch = fs->filterInfo->origMatch;
-		    scalefilterFiniFilterInfo (s, TRUE);
-		}
-		fs->matchApplied = FALSE;
-	    }
+	    fs->matchApplied = FALSE;
 	}
     }
 }
@@ -738,27 +739,17 @@ scalefilterScreenOptionChanged (CompScreen               *s,
 
 static Bool
 scalefilterInitDisplay (CompPlugin  *p,
-	    		CompDisplay *d)
+			CompDisplay *d)
 {
     ScaleFilterDisplay *fd;
-    CompPlugin         *scale = findActivePlugin ("scale");
-    CompOption         *option;
-    int                nOption;
 
-    if (!scale || !scale->vTable->getDisplayOptions)
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
 	return FALSE;
 
-    option = (*scale->vTable->getDisplayOptions) (scale, d, &nOption);
-
-    if (getIntOptionNamed (option, nOption, "abi", 0) != SCALE_ABIVERSION)
-    {
-	compLogMessage (d, "scalefilter", CompLogLevelError,
-			"scale ABI version mismatch");
+    if (!checkPluginABI ("scale", SCALE_ABIVERSION))
 	return FALSE;
-    }
 
-    scaleDisplayPrivateIndex = getIntOptionNamed (option, nOption, "index", -1);
-    if (scaleDisplayPrivateIndex < 0)
+    if (!getPluginDisplayIndex (d, "scale", &scaleDisplayPrivateIndex))
 	return FALSE;
 
     fd = malloc (sizeof (ScaleFilterDisplay));
@@ -785,10 +776,15 @@ scalefilterInitDisplay (CompPlugin  *p,
     if (fd->xic)
 	setlocale (LC_CTYPE, "");
 
+    fd->textAvailable = checkPluginABI ("text", TEXT_ABIVERSION);
+    if (!fd->textAvailable)
+	compLogMessage (d, "scalefilter", CompLogLevelWarn,
+			"No compatible text plugin found.");
+
     WRAP (fd, d, handleEvent, scalefilterHandleEvent);
     WRAP (fd, d, handleCompizEvent, scalefilterHandleCompizEvent);
 
-    d->privates[displayPrivateIndex].ptr = fd;
+    d->base.privates[displayPrivateIndex].ptr = fd;
 
     return TRUE;
 }
@@ -814,7 +810,7 @@ scalefilterFiniDisplay (CompPlugin  *p,
 
 static Bool
 scalefilterInitScreen (CompPlugin *p,
-	    	       CompScreen *s)
+		       CompScreen *s)
 {
     ScaleFilterScreen *fs;
 
@@ -838,14 +834,14 @@ scalefilterInitScreen (CompPlugin *p,
     scalefilterSetFontColorNotify (s, scalefilterScreenOptionChanged);
     scalefilterSetBackColorNotify (s, scalefilterScreenOptionChanged);
 
-    s->privates[fd->screenPrivateIndex].ptr = fs;
+    s->base.privates[fd->screenPrivateIndex].ptr = fs;
 
     return TRUE;
 }
 
 static void
 scalefilterFiniScreen (CompPlugin *p,
-	    	       CompScreen *s)
+		       CompScreen *s)
 {
     FILTER_SCREEN (s);
     SCALE_SCREEN (s);
@@ -862,6 +858,32 @@ scalefilterFiniScreen (CompPlugin *p,
     free (fs);
 }
 
+static CompBool
+scalefilterInitObject (CompPlugin *p,
+		       CompObject *o)
+{
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) scalefilterInitDisplay,
+	(InitPluginObjectProc) scalefilterInitScreen
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+scalefilterFiniObject (CompPlugin *p,
+		       CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) scalefilterFiniDisplay,
+	(FiniPluginObjectProc) scalefilterFiniScreen
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
+}
+
 static Bool
 scalefilterInit (CompPlugin *p)
 {
@@ -878,27 +900,13 @@ scalefilterFini (CompPlugin *p)
     freeDisplayPrivateIndex (displayPrivateIndex);
 }
 
-static int
-scalefilterGetVersion (CompPlugin *plugin,
-	    	       int	 version)
-{
-    return ABIVERSION;
-}
-
 CompPluginVTable scalefilterVTable = {
     "scalefilter",
-    scalefilterGetVersion,
     0,
     scalefilterInit,
     scalefilterFini,
-    scalefilterInitDisplay,
-    scalefilterFiniDisplay,
-    scalefilterInitScreen,
-    scalefilterFiniScreen,
-    0,
-    0,
-    0,
-    0,
+    scalefilterInitObject,
+    scalefilterFiniObject,
     0,
     0
 };
