@@ -33,7 +33,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <compiz.h>
+#include <compiz-core.h>
 #include "showdesktop_options.h"
 
 #include <math.h>
@@ -110,22 +110,23 @@ typedef struct _ShowdesktopWindow
 } ShowdesktopWindow;
 
 /* shortcut macros, usually named X_DISPLAY, X_SCREEN and X_WINDOW
- * these might seem overly complicated but they are shortcuts so we don't have to access the privates arrays all the time
+ * these might seem overly complicated but they are shortcuts so 
+ * we don't have to access the privates arrays all the time
  * */
 #define GET_SHOWDESKTOP_DISPLAY(d)				     \
-    ((ShowdesktopDisplay *) (d)->privates[displayPrivateIndex].ptr)
+    ((ShowdesktopDisplay *) (d)->base.privates[displayPrivateIndex].ptr)
 
 #define SD_DISPLAY(d)			   \
     ShowdesktopDisplay *sd = GET_SHOWDESKTOP_DISPLAY (d)
 
 #define GET_SHOWDESKTOP_SCREEN(s, sd)					 \
-    ((ShowdesktopScreen *) (s)->privates[(sd)->screenPrivateIndex].ptr)
+    ((ShowdesktopScreen *) (s)->base.privates[(sd)->screenPrivateIndex].ptr)
 
 #define SD_SCREEN(s)							\
     ShowdesktopScreen *ss = GET_SHOWDESKTOP_SCREEN (s, GET_SHOWDESKTOP_DISPLAY (s->display))
 
 #define GET_SHOWDESKTOP_WINDOW(w, ss)					  \
-    ((ShowdesktopWindow *) (w)->privates[(ss)->windowPrivateIndex].ptr)
+    ((ShowdesktopWindow *) (w)->base.privates[(ss)->windowPrivateIndex].ptr)
 
 #define SD_WINDOW(w)					       \
     ShowdesktopWindow *sw = GET_SHOWDESKTOP_WINDOW  (w,		       \
@@ -139,12 +140,19 @@ static int displayPrivateIndex;
 static Bool
 isSDWin (CompWindow *w)
 {
+    if (w->grabbed)
+	return FALSE;
+
     if (!(*w->screen->focusWindow) (w))
 	return FALSE;
 
     if (!matchEval (showdesktopGetWindowMatch (w->screen), w))
 	return FALSE;
 
+    if (w->wmType & (CompWindowTypeDesktopMask |
+		     CompWindowTypeDockMask))
+	return FALSE;
+
     if (w->state & CompWindowStateSkipPagerMask)
 	return FALSE;
 
@@ -176,7 +184,6 @@ setSDWindowHints (CompWindow *w,
     }
 
     changeWindowState (w, state);
-    recalcWindowActions (w);
 
     if (enterSDMode)
     {
@@ -746,6 +753,9 @@ showdesktopInitDisplay (CompPlugin  *p,
 {
     ShowdesktopDisplay *sd;
 
+    if (!checkPluginABI ("core", CORE_ABIVERSION))
+	return FALSE;
+
     sd = malloc (sizeof (ShowdesktopDisplay));
     if (!sd)
 	return FALSE;
@@ -759,7 +769,7 @@ showdesktopInitDisplay (CompPlugin  *p,
 
     WRAP (sd, d, handleEvent, showdesktopHandleEvent);
 
-    d->privates[displayPrivateIndex].ptr = sd;
+    d->base.privates[displayPrivateIndex].ptr = sd;
 
     return TRUE;
 }
@@ -809,7 +819,7 @@ showdesktopInitScreen (CompPlugin *p,
     WRAP (ss, s, getAllowedActionsForWindow,
 	  showdesktopGetAllowedActionsForWindow);
 
-    s->privates[sd->screenPrivateIndex].ptr = ss;
+    s->base.privates[sd->screenPrivateIndex].ptr = ss;
 
     return TRUE;
 }
@@ -859,7 +869,7 @@ showdesktopInitWindow (CompPlugin *p,
     sw->stateMask      = 0;
     sw->notAllowedMask = 0;
 
-    w->privates[ss->windowPrivateIndex].ptr = sw;
+    w->base.privates[ss->windowPrivateIndex].ptr = sw;
 
     return TRUE;
 }
@@ -874,30 +884,44 @@ showdesktopFiniWindow (CompPlugin *p,
     free (sw);
 }
 
-static int
-showdesktopGetVersion (CompPlugin *plugin,
-		       int        version)
+static CompBool
+showdesktopInitObject (CompPlugin *p,
+		       CompObject *o)
 {
-    return ABIVERSION;
+    static InitPluginObjectProc dispTab[] = {
+	(InitPluginObjectProc) 0, /* InitCore */
+	(InitPluginObjectProc) showdesktopInitDisplay,
+	(InitPluginObjectProc) showdesktopInitScreen,
+	(InitPluginObjectProc) showdesktopInitWindow
+    };
+
+    RETURN_DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), TRUE, (p, o));
+}
+
+static void
+showdesktopFiniObject (CompPlugin *p,
+		       CompObject *o)
+{
+    static FiniPluginObjectProc dispTab[] = {
+	(FiniPluginObjectProc) 0, /* FiniCore */
+	(FiniPluginObjectProc) showdesktopFiniDisplay,
+	(FiniPluginObjectProc) showdesktopFiniScreen,
+	(FiniPluginObjectProc) showdesktopFiniWindow
+    };
+
+    DISPATCH (o, dispTab, ARRAY_SIZE (dispTab), (p, o));
 }
 
 /* plugin vtable */
 static CompPluginVTable showdesktopVTable = {
     "showdesktop",
-    showdesktopGetVersion,
     0,
     showdesktopInit,
     showdesktopFini,
-    showdesktopInitDisplay,
-    showdesktopFiniDisplay,
-    showdesktopInitScreen,
-    showdesktopFiniScreen,
-    showdesktopInitWindow,
-    showdesktopFiniWindow,
-    NULL,
-    NULL,
-    NULL,
-    NULL
+    showdesktopInitObject,
+    showdesktopFiniObject,
+    0,
+    0
 };
 
 /* send plugin info */
